package remotestocks;

public class RemoteStockCallback implements stocks.StockQuoteClient
{

// Client is the CORBA client whose methods we want to invoke
	protected StockQuoteClientRef client;

// Server is the CORBA-wrapper for the stock quote server. If there
// is an error sending a quote to the client, we tell the wrapper
// about it so it can remove this client from its tables.
	protected RemoteStockQuoteImpl server;

// makeRemoteStockQuote converts a stocks.StockQuote object (the Non-CORBA
// object) into a remotestocks.StockQuote object (the CORBA version of
// the stock quote).

	protected static StockQuote makeRemoteStockQuote(
		stocks.StockQuote quote)
	{
		return new StockQuote(quote.stock, quote.amount,
			quote.change);
	}

	public RemoteStockCallback(StockQuoteClientRef client,
		RemoteStockQuoteImpl server)
	{
		this.client = client;
		this.server = server;
	}

// quote is invoked by the real stock quote server implementation, not
// the CORBA server.

	public void quote(stocks.StockQuote quote) throws
		 stocks.StockQuoteClientException
	{
// Try to invoke the remote method, converting the stocks.StockQuote
// to the CORBA version, remotestocks.StockQuote
		try {
			client.quote(makeRemoteStockQuote(quote));
		} catch (Exception e) {

// If there was an error, remove this client from the server, then
// throw an exception to the real implementation server.

			try {
				server.removeCallback(this);
			} catch (Exception ignore) {
			}
			throw new stocks.StockQuoteClientException();
		}
	}
}
