package banking;

import java.util.*;

public class BankingImpl extends Object implements Banking
{
	public Hashtable accountTable;

	public BankingImpl()
	{
		accountTable = new Hashtable();

		accountTable.put(
			new Account("AA1234", "1017", Account.CHECKING),
			new Integer(50000));	// $500.00 balance

		accountTable.put(
			new Account("AA1234", "1017", Account.SAVINGS),
			new Integer(148756));	// $1487.56 balance

		accountTable.put(
			new Account("AB5678", "4456", Account.CHECKING),
			new Integer(7742));	// $77.32 balance

		accountTable.put(
			new Account("AB5678", "4456", Account.SAVINGS),
			new Integer(32201));	// $322.01 balance

		System.out.println("Bank accounts:");
		Enumeration e = accountTable.keys();
		while (e.hasMoreElements()) {
			System.out.println(e.nextElement());
		}
	}

	public int getBalance(Account account)
	throws BankingException
	{
		System.out.println("Got balance request for: "+account);
		Integer balance = (Integer) accountTable.get(account);

		if (balance == null) {
			Enumeration e = accountTable.keys();
			while (e.hasMoreElements()) {
				Account a = (Account)e.nextElement();
				System.out.println(a);
				System.out.println(a.equals(account));
			}
			throw new InvalidAccountException(account);
		}

		return balance.intValue();
	}

	public synchronized void withdraw(Account account, int amount)
	throws BankingException
	{
		Integer balance = (Integer) accountTable.get(account);

		if (balance == null) {
			throw new InvalidAccountException(account);
		}

		if (balance.intValue() < amount) {
			throw new InsufficientFundsException();
		}

		accountTable.put(account, new Integer(balance.intValue() -
			amount));
	}

	public synchronized void deposit(Account account, int amount)
	throws BankingException
	{
		Integer balance = (Integer) accountTable.get(account);

		if (balance == null) {
			throw new InvalidAccountException(account);
		}

		accountTable.put(account, new Integer(balance.intValue() +
			amount));
	}

	public synchronized void transfer(Account fromAccount,
		Account toAccount, int amount)
	throws BankingException
	{
		Integer fromBalance = (Integer) accountTable.get(fromAccount);

		if (fromBalance == null) {
			throw new InvalidAccountException(fromAccount);
		}

		Integer toBalance = (Integer) accountTable.get(toAccount);

		if (toBalance == null) {
			throw new InvalidAccountException(toAccount);
		}

		if (fromBalance.intValue() < amount) {
			throw new InsufficientFundsException();
		}

		
		accountTable.put(fromAccount,
			new Integer(fromBalance.intValue() - amount));
		accountTable.put(toAccount,
			new Integer(toBalance.intValue() + amount));
	}
}

