package remotebanking;

import java.util.*;

public class RemoteBankingImpl implements BankingServant
{

// Create a banking.Account from an AccountInfo object

	public static banking.Account makeAccount(AccountInfo info)
	{
		return new banking.Account(info.id, info.password,
			info.which);
	}

// Create an AccountInfo object from a banking.Account object

	public static AccountInfo makeAccountInfo(banking.Account account)
	{
		return new AccountInfo(account.id, account.password,
			account.which);
	}

	protected banking.BankingImpl impl;

	public RemoteBankingImpl(banking.BankingImpl impl)
	{
		this.impl = impl;
	}

	public int getBalance(AccountInfo accountInfo)
	throws sunw.corba.SystemException, InvalidAccountException,
		BankingException
	{
		try {
			int balance = impl.getBalance(
				makeAccount(accountInfo));
			return balance;
		} catch (banking.InvalidAccountException excep) {
			throw new InvalidAccountException(
				makeAccountInfo(excep.account));
		} catch (banking.BankingException e) {
			throw new BankingException();
		}
	}
					
// call the withdraw function in the real implementation, catching
// any exceptions and throwing the equivalent CORBA exception

	public synchronized void withdraw(AccountInfo accountInfo, int amount)
	throws sunw.corba.SystemException, InvalidAccountException,
		InsufficientFundsException, BankingException
	{

		try {

// Call the real withdraw method, converting the accountInfo object
// to a banking.Account object first

			impl.withdraw( makeAccount(accountInfo), amount);

		} catch (banking.InvalidAccountException excep) {

// The banking.InvalidAccountException contains an Account object.
// Convert it to an AccountInfo object when throwing the CORBA exception

			throw new InvalidAccountException(
				makeAccountInfo(excep.account));

		} catch (banking.InsufficientFundsException nsf) {
			throw new InsufficientFundsException();
		} catch (banking.BankingException e) {
			throw new BankingException();
		}

	}

	public synchronized void deposit(AccountInfo accountInfo, int amount)
	throws sunw.corba.SystemException, InvalidAccountException,
		BankingException
	{
		try {
			impl.deposit(makeAccount(accountInfo), amount);

		} catch (banking.InvalidAccountException excep) {
			throw new InvalidAccountException(
				makeAccountInfo(excep.account));
		} catch (banking.BankingException e) {
			throw new BankingException();
		}
	}

// Transfer subtracts an amount from fromAccount and adds it to toAccount.
// If either account is invalid it throws InvalidAccountException.
// If there isn't enough money in fromAccount it throws
// InsufficientFundsException.

	public synchronized void transfer(AccountInfo fromAccountInfo,
		AccountInfo toAccountInfo, int amount)
	throws sunw.corba.SystemException, InvalidAccountException,
		InsufficientFundsException, BankingException
	{
		try {
			impl.transfer(
				makeAccount(fromAccountInfo),
				makeAccount(toAccountInfo),
				amount);
		} catch (banking.InvalidAccountException excep) {
			throw new InvalidAccountException(
				makeAccountInfo(excep.account));
		} catch (banking.InsufficientFundsException nsf) {
			throw new InsufficientFundsException();
		} catch (banking.BankingException e) {
			throw new BankingException();
		}
	}
}

